// =============================================================================
// Copyright (C) MACNICA inc.
// =============================================================================
// FILE NAME	: avb_frm_wt_pkt_anls.v
// DEPARTMENT	: MACNICA Strategic Technology Group
// AUTHOR		: Ryosuke Yamamoto
// DATE			: 03/10/2014
// =============================================================================
// DESCRIPTION	: 
//
// =============================================================================
// REVISION HISTORY	:
//	v1.0	: Initial version release
//
// =============================================================================
// PARAMETERS	:
//
// =============================================================================
// I/O PORTS	:
//
// =============================================================================
`timescale 1ps/1ps
`default_nettype none

module	avb_frm_wt_pkt_anls (
	clk						,
	rst_n					,
	srst					,

	valid_i					,
	startofpacket_i			,
	endofpacket_i			,
	data_i					,
	ready_o					,

	src_mac_adr_i			,
	frame_number_o			,
	packet_number_o			,
	total_packet_count_o	,
	time_stamp_sec_o		,
	time_stamp_ns_o			,
	fifo_wt_req_o			,
	fifo_wt_start_o			,
	fifo_wt_end_o			,

	fifo_sel_i				,
	fifo0_full_i			,
	fifo0_afull_i			,
	fifo0_we_o				,
	fifo1_full_i			,
	fifo1_afull_i			,
	fifo1_we_o				,
	fifo_wdata_o			
	
) ;
	
// =============================================================================
// PARAMETER DEFINITION
// =============================================================================

	// ---------------------------------------------------------------------
	// Below parameters have to be defined from upper module
	// ---------------------------------------------------------------------

	// ---------------------------------------------------------------------
	// Please do not change the following parameters
	// ---------------------------------------------------------------------
	parameter	ST_DATA_BIT		= 64 ;				// Avalon-ST data bit width
	localparam	WFIFO_DATA_BIT	= ST_DATA_BIT * 2 ;	// FIFO data bit width

	parameter	MAC_ADR_BIT		= 48 ;
	parameter	TAG_BIT			= 32 ;
	parameter	TYPE_BIT		= 16 ;
	parameter	FRM_NUM_BIT		= 16 ;
	parameter	PACKET_NUM_BIT	= 32 ;
	parameter	T_STP_SEC_BIT	= 48 ;
	parameter	T_STP_NS_BIT	= 32 ;

	localparam	ST_IDLE			= 4'h0 ;
	localparam	ST_HEAD1		= 4'h1 ;
	localparam	ST_HEAD2		= 4'h2 ;
	localparam	ST_HEAD3		= 4'h3 ;
	localparam	ST_HEAD4		= 4'h4 ;
	localparam	ST_HEAD_DATA	= 4'h5 ;
	localparam	ST_DATA1		= 4'h6 ;
	localparam	ST_DATA2		= 4'h7 ;
	localparam	ST_DATA3		= 4'h8 ;
	

// =============================================================================
// PORT DECLARATION
// =============================================================================

	input	wire								clk						;
	input	wire								rst_n					;
	input	wire								srst					;

	input	wire								valid_i					;
	input	wire								startofpacket_i			;
	input	wire								endofpacket_i			;
	input	wire	[    ST_DATA_BIT-1 : 0 ]	data_i					;
	output	wire								ready_o					;
	
	input	wire	[    MAC_ADR_BIT-1 : 0 ]	src_mac_adr_i			;
	output	wire	[    FRM_NUM_BIT-1 : 0 ]	frame_number_o			;
	output	wire	[ PACKET_NUM_BIT-1 : 0 ]	packet_number_o			;
	output	wire	[ PACKET_NUM_BIT-1 : 0 ]	total_packet_count_o	;
	output	wire	[  T_STP_SEC_BIT-1 : 0 ]	time_stamp_sec_o		;
	output	wire	[   T_STP_NS_BIT-1 : 0 ]	time_stamp_ns_o			;
	output	wire								fifo_wt_req_o			;
	output	wire								fifo_wt_start_o			;
	output	wire								fifo_wt_end_o			;

	input	wire	[                1 : 0 ]	fifo_sel_i				;
	input	wire								fifo0_full_i			;
	input	wire								fifo0_afull_i			;
	output	wire								fifo0_we_o				;
	input	wire								fifo1_full_i			;
	input	wire								fifo1_afull_i			;
	output	wire								fifo1_we_o				;
	output	wire	[ WFIFO_DATA_BIT-1 : 0 ]	fifo_wdata_o			;


// =============================================================================
// REG / WIRE DECLARATION
// =============================================================================

	wire								sop					;
	wire								eop					;
	wire								state_en			;

	reg		[                3 : 0 ]	state_ff			;
	reg									ready_ff			;
	reg		[    MAC_ADR_BIT-1 : 0 ]	dst_adr_ff			;	// not use
	reg		[    MAC_ADR_BIT-1 : 0 ]	src_adr_ff			;
	reg		[        TAG_BIT-1 : 0 ]	tag_ff				;
	reg		[       TYPE_BIT-1 : 0 ]	ether_type_ff		;	// not use
	reg		[    FRM_NUM_BIT-1 : 0 ]	frame_num_ff		;
	reg		[ PACKET_NUM_BIT-1 : 0 ]	pkt_num_ff			;
	reg		[ PACKET_NUM_BIT-1 : 0 ]	ttl_pkt_count_ff	;
	reg		[  T_STP_SEC_BIT-1 : 0 ]	time_stamp_sec_ff	;
	reg		[   T_STP_NS_BIT-1 : 0 ]	time_stamp_ns_ff	;
	reg									fifo_wt_req_ff		;
	reg									fifo_wt_start_ff	;
	reg									fifo_wt_end_ff		;
	reg		[                1 : 0 ]	fifo_we_ff			;
	reg		[ WFIFO_DATA_BIT-1 : 0 ]	fifo_wdata_ff		;


// =============================================================================
// FUNCTION DESCRIPTION
// =============================================================================


	assign	sop	= valid_i & startofpacket_i ;
	assign	eop	= valid_i & endofpacket_i ;

	assign	state_en	= ( ( state_ff == ST_IDLE ) 
						  | ( state_ff == ST_HEAD1 ) 
						  | ( state_ff == ST_HEAD2 ) 
						  | ( state_ff == ST_HEAD3 ) 
						  | ( state_ff == ST_HEAD4 ) 
						  | ( ( ( fifo_sel_i[0] & !fifo0_afull_i ) 
						      | ( fifo_sel_i[1] & !fifo1_afull_i ) ) ) ) ;

	always @ ( posedge clk or negedge rst_n ) begin
		if ( !rst_n ) begin
			state_ff			<= ST_IDLE ;
			ready_ff			<= 1'b0 ;
			dst_adr_ff			<= { MAC_ADR_BIT{1'b0} } ;
			src_adr_ff			<= { MAC_ADR_BIT{1'b0} } ;
			tag_ff				<= { TAG_BIT{1'b0} } ;
			ether_type_ff		<= { TYPE_BIT{1'b0} } ;
			frame_num_ff		<= { FRM_NUM_BIT{1'b0} } ;
			pkt_num_ff			<= { PACKET_NUM_BIT{1'b0} } ;
			ttl_pkt_count_ff	<= { PACKET_NUM_BIT{1'b0} } ;
			time_stamp_sec_ff	<= { T_STP_SEC_BIT{1'b0} } ;
			time_stamp_ns_ff	<= { T_STP_NS_BIT{1'b0} } ;
			fifo_wt_req_ff		<= 1'b0 ;
			fifo_wt_start_ff	<= 1'b0 ;
			fifo_wt_end_ff		<= 1'b0 ;
			fifo_we_ff			<= 1'b0 ;
			fifo_wdata_ff		<= { WFIFO_DATA_BIT{1'b0} } ;
		end
		else if ( srst ) begin
			state_ff			<= ST_IDLE ;
			ready_ff			<= 1'b0 ;
			dst_adr_ff			<= { MAC_ADR_BIT{1'b0} } ;
			src_adr_ff			<= { MAC_ADR_BIT{1'b0} } ;
			tag_ff				<= { TAG_BIT{1'b0} } ;
			ether_type_ff		<= { TYPE_BIT{1'b0} } ;
			frame_num_ff		<= { FRM_NUM_BIT{1'b0} } ;
			pkt_num_ff			<= { PACKET_NUM_BIT{1'b0} } ;
			ttl_pkt_count_ff	<= { PACKET_NUM_BIT{1'b0} } ;
			time_stamp_sec_ff	<= { T_STP_SEC_BIT{1'b0} } ;
			time_stamp_ns_ff	<= { T_STP_NS_BIT{1'b0} } ;
			fifo_wt_req_ff		<= 1'b0 ;
			fifo_wt_start_ff	<= 1'b0 ;
			fifo_wt_end_ff		<= 1'b0 ;
			fifo_we_ff			<= 1'b0 ;
			fifo_wdata_ff		<= { WFIFO_DATA_BIT{1'b0} } ;
		end
		else begin
			if ( state_en ) begin
				ready_ff			<= 1'b1 ;
				case ( state_ff ) 
					ST_IDLE		: begin
						if ( eop ) begin
							state_ff		<= ST_IDLE ;
						end
						else if ( sop ) begin
							state_ff		<= ST_HEAD1 ;
							dst_adr_ff		<= data_i[MAC_ADR_BIT-1:0] ;
							fifo_wt_req_ff	<= 1'b1 ;
						end
						else begin
							dst_adr_ff		<= { MAC_ADR_BIT{1'b0} } ;
							src_adr_ff		<= { MAC_ADR_BIT{1'b0} } ;
							tag_ff			<= { TAG_BIT{1'b0} } ;
							fifo_wt_req_ff	<= 1'b0 ;
						end
						ether_type_ff		<= ether_type_ff ;
						frame_num_ff		<= frame_num_ff ;
						pkt_num_ff			<= pkt_num_ff ;
						ttl_pkt_count_ff	<= ttl_pkt_count_ff ;
						time_stamp_sec_ff	<= time_stamp_sec_ff ;
						time_stamp_ns_ff	<= time_stamp_ns_ff ;
						fifo_wt_start_ff	<= 1'b0 ;
						fifo_we_ff			<= 1'b0 ;
						fifo_wdata_ff		<= { WFIFO_DATA_BIT{1'b0} } ;
						fifo_wt_end_ff		<= 1'b0 ;
					end

					ST_HEAD1	: begin
						if ( valid_i ) begin
							if ( eop ) begin
								state_ff		<= ST_IDLE ;
								fifo_wt_req_ff	<= 1'b0 ;
							end
							else begin
								state_ff	<= ST_HEAD2 ;
							end
							src_adr_ff					<= data_i[ST_DATA_BIT  -1:ST_DATA_BIT/4] ;
							tag_ff[TAG_BIT-1:TAG_BIT/2]	<= data_i[ST_DATA_BIT/4-1:0] ;
						end
					end

					ST_HEAD2	: begin
						if ( valid_i ) begin
							if ( eop ) begin
								state_ff		<= ST_IDLE ;
								fifo_wt_req_ff	<= 1'b0 ;
							end
							else begin
								state_ff	<= ST_HEAD3 ;
							end
							tag_ff[TAG_BIT/2-1:0]							<= data_i[ST_DATA_BIT    -1:ST_DATA_BIT*3/4] ;
							ether_type_ff									<= data_i[ST_DATA_BIT*3/4-1:ST_DATA_BIT*2/4] ;
							frame_num_ff									<= data_i[ST_DATA_BIT*2/4-1:ST_DATA_BIT*1/4] ;
							pkt_num_ff[PACKET_NUM_BIT-1:PACKET_NUM_BIT/2]	<= data_i[ST_DATA_BIT*1/4-1:0] ;
						end
					end

					ST_HEAD3	: begin
						if ( valid_i ) begin
							if ( eop ) begin
								state_ff		<= ST_IDLE ;
								fifo_wt_req_ff	<= 1'b0 ;
							end
							else begin
								state_ff	<= ST_HEAD4 ;
							end
							pkt_num_ff[PACKET_NUM_BIT/2-1:0]						<= data_i[ST_DATA_BIT    -1:ST_DATA_BIT*3/4] ;
							ttl_pkt_count_ff										<= data_i[ST_DATA_BIT*3/4-1:ST_DATA_BIT*1/4] ;
							time_stamp_sec_ff[T_STP_SEC_BIT-1:T_STP_SEC_BIT*2/3]	<= data_i[ST_DATA_BIT*1/4-1:0] ;
						end
					end

					ST_HEAD4	: begin
						if ( valid_i ) begin
							if ( src_adr_ff == src_mac_adr_i ) begin
								state_ff			<= ST_DATA1 ;
								fifo_wt_start_ff	<= 1'b1 ;
							end
							else begin
								state_ff		<= ST_IDLE ;
								fifo_wt_req_ff	<= 1'b0 ;
							end
							time_stamp_sec_ff[T_STP_SEC_BIT*2/3-1:0]	<= data_i[ST_DATA_BIT  -1:ST_DATA_BIT/2] ;
							time_stamp_ns_ff							<= data_i[ST_DATA_BIT/2-1:0] ;
						end
					end

					ST_DATA1		: begin
						fifo_wt_start_ff	<= 1'b0 ;
						if ( valid_i ) begin
							if ( eop ) begin
								state_ff		<= ST_IDLE ;
								fifo_we_ff		<= fifo_sel_i & 2'b11 ;
								fifo_wt_end_ff	<= 1'b1 ;
							end
							else begin
								state_ff	<= ST_DATA2 ;
								fifo_we_ff	<= 2'b00 ;
							end
							fifo_wdata_ff[15: 0]	<= data_i[ST_DATA_BIT    -1:ST_DATA_BIT*3/4] ;
							fifo_wdata_ff[31:16]	<= data_i[ST_DATA_BIT*3/4-1:ST_DATA_BIT*2/4] ;
							fifo_wdata_ff[47:32]	<= data_i[ST_DATA_BIT*2/4-1:ST_DATA_BIT*1/4] ;
							fifo_wdata_ff[63:48]	<= data_i[ST_DATA_BIT*1/4-1:0] ;
						end
						else begin
							fifo_we_ff	<= 2'b00 ;
						end
					end

					ST_DATA2		: begin
						if ( valid_i ) begin
							if ( eop ) begin
								state_ff		<= ST_IDLE ;
								fifo_wt_end_ff	<= 1'b1 ;
							end
							else begin
								state_ff	<= ST_DATA1 ;
							end
							fifo_wdata_ff[ 79: 64]	<= data_i[ST_DATA_BIT    -1:ST_DATA_BIT*3/4] ;
							fifo_wdata_ff[ 95: 80]	<= data_i[ST_DATA_BIT*3/4-1:ST_DATA_BIT*2/4] ;
							fifo_wdata_ff[111: 96]	<= data_i[ST_DATA_BIT*2/4-1:ST_DATA_BIT*1/4] ;
							fifo_wdata_ff[127:112]	<= data_i[ST_DATA_BIT*1/4-1:0] ;
							fifo_we_ff				<= fifo_sel_i & 2'b11 ;
						end
						else begin
							fifo_we_ff	<= 2'b00 ;
						end
					end

					default			: begin
						state_ff	<= ST_IDLE ;
					end
				endcase

			end
			else begin
				ready_ff	<= 1'b0 ;
			end

		end
	end

	assign	ready_o					= ready_ff ;

	assign	frame_number_o			= frame_num_ff ;
	assign	packet_number_o			= pkt_num_ff ;
	assign	total_packet_count_o	= ttl_pkt_count_ff ;
	assign	time_stamp_sec_o		= time_stamp_sec_ff ;
	assign	time_stamp_ns_o			= time_stamp_ns_ff ;
	assign	fifo_wt_req_o			= fifo_wt_req_ff ;
	assign	fifo_wt_start_o			= fifo_wt_start_ff ;
	assign	fifo_wt_end_o			= fifo_wt_end_ff ;


	assign	fifo0_we_o				= fifo_we_ff[0] ;
	assign	fifo1_we_o				= fifo_we_ff[1] ;
	assign	fifo_wdata_o			= fifo_wdata_ff ;


endmodule

`default_nettype wire
