// =============================================================================
// Copyright (C) MACNICA inc.
// =============================================================================
// FILE NAME	: avb_pkt_asm.v
// DEPARTMENT	: MACNICA Strategic Technology Group
// AUTHOR		: Ryosuke Yamamoto
// DATE 		: 03/05/2014
// =============================================================================
// DESCRIPTION	:
//		
// =============================================================================
// REVISION HISTORY :
//		v1.0		: Initial Release
//
// =============================================================================
// PARAMETERS	:
//		Q_BIT		: Quantized bit width per color plane
//		PLANE		: Color plane count
//		SIZE_BIT	: Size setting signal bit width
//
// =============================================================================
// I/O PORTS	:
//		clk					: Clock for all circuit
//		rst_n				: Asynchronous reset ( low active )
//
//		valid_i				: Data valid of input VSI frame data.
//		frame_start_i		: Frame start of input VSI frame data.
//		frame_end_i			: Frame end of input VSI frame data.
//		field_i				: Field status of input VSI frame data.
//							  0:Odd field  1:Even field
//		pixel_i				: Pixel data of input VSI frame data.
//		width_i				: Horizontal size of input VSI frame data
//		height_i			: Vertical size of input frame data
//		ready_o				: Data reception ready of input VSI frame data.
//
//		valid_o				: Data valid of output VSI frame data.
//		frame_start_o		: Frame start of output VSI frame data.
//		frame_end_o			: Frame end of output VSI frame data.
//		field_o				: Field status of output VSI frame data.
//							  0:Odd field  1:Even field
//		pixel_o				: Pixel data of output VSI frame data.
//		width_o				: Horizontal size of output VSI frame data
//		height_o			: Vertical size of output frame data
//		ready_i				: Data reception ready of output VSI frame data.
//
//
// =============================================================================
`timescale 1ps/1ps
`default_nettype none

module avb_pkt_asm (
	clk							,
	rst_n						,

	avm_address_i				,
	avm_byteenable_i			,
	avm_read_i					,
	avm_write_i					,
	avm_readdatavalid_o			,
	avm_readdata_o				,
	avm_writedata_i				,
	avm_waitrequest_o			,

	local_sync_256ns_pulse_i	,
	master_clock_sec_i			,
	master_clock_ns_i			,

	valid_i						,
	startofpacket_i				,
	endofpacket_i				,
	data_i						,
	ready_o						,

	tx_valid_o					,
	tx_startofpacket_o			,
	tx_endofpacket_o			,
	tx_data_o					,
	tx_mod_o					,
	tx_err_o					,
	tx_ready_i					

) ;


// =============================================================================
// PARAMETER DEFINITION
// =============================================================================

	// ---------------------------------------------------------------------
	// Below parameters have to be defined from upper module
	// ---------------------------------------------------------------------
	parameter	Q_BIT			= 8 ;
	parameter	PLANE			= 2 ;
	parameter	SIZE_BIT		= 11 ;

	// ---------------------------------------------------------------------
	// Please do not change the following parameters
	// ---------------------------------------------------------------------
	parameter	PIXEL_BIT		= Q_BIT * PLANE ;
	parameter	DATA_BIT		= 32 ;
	parameter	T_STP_SEC_BIT	= 48 ;
	parameter	T_STP_NS_BIT	= 32 ;
	parameter	PKT_SIZE_BIT	= 11 ;
	parameter	T_PKT_SIZE_BIT	= SIZE_BIT * 2 ;
	parameter	MAC_ADR_BIT		= 48 ;
	parameter	TAG_BIT			= 32 ;
	parameter	TYPE_BIT		= 16 ;
	parameter	FRM_NUM_BIT		= 16 ;
	parameter	PACKET_NUM_BIT	= 32 ;

// =============================================================================
// PORT DECLARATION
// =============================================================================

	input	wire							clk							;
	input	wire							rst_n						;

	input	wire	[               3 : 0 ]	avm_address_i				;
	input	wire	[               3 : 0 ]	avm_byteenable_i			;
	input	wire							avm_read_i					;
	input	wire							avm_write_i					;
	output	wire							avm_readdatavalid_o			;
	output	wire	[              31 : 0 ]	avm_readdata_o				;
	input	wire	[              31 : 0 ]	avm_writedata_i				;
	output	wire							avm_waitrequest_o			;

	input	wire	[               1 : 0 ]	local_sync_256ns_pulse_i	; //   (b0) local clock toggles every 256 synchronized ns.  (b1) if as synced
	input	wire	[ T_STP_SEC_BIT-1 : 0 ]	master_clock_sec_i			; // master time used to gate start of audio playback
	input	wire	[  T_STP_NS_BIT-1 : 0 ]	master_clock_ns_i			; //

	input	wire							valid_i						;
	input	wire							startofpacket_i				;
	input	wire							endofpacket_i				;
	input	wire	[   Q_BIT*PLANE-1 : 0 ]	data_i						;
	output	wire							ready_o						;

	output	wire							tx_valid_o					;
	output	wire							tx_startofpacket_o			;
	output	wire							tx_endofpacket_o			;
	output	wire	[      DATA_BIT-1 : 0 ]	tx_data_o					;
	output	wire	[               1 : 0 ]	tx_mod_o					;
	output	wire							tx_err_o					;
	input	wire							tx_ready_i					;


// =============================================================================
// REG / WIRE DECLARATION
// =============================================================================

	wire								csr_srst			;
	wire	[   PKT_SIZE_BIT-1 : 0 ]	csr_packet_size		;
	wire	[    MAC_ADR_BIT-1 : 0 ]	csr_des_mac_adr		;
	wire	[    MAC_ADR_BIT-1 : 0 ]	csr_src_mac_adr		;
	wire	[        TAG_BIT-1 : 0 ]	csr_tag				;
	wire	[       TYPE_BIT-1 : 0 ]	csr_ether_type		;
	wire								csr_packet_transmit	;

	wire								avs2vsi_valid		;
	wire								avs2vsi_frame_start	;
	wire								avs2vsi_frame_end	;
	wire								avs2vsi_field		;
	wire	[      PIXEL_BIT-1 : 0 ]	avs2vsi_data		;
	wire	[       SIZE_BIT-1 : 0 ]	avs2vsi_width		;
	wire	[       SIZE_BIT-1 : 0 ]	avs2vsi_height		;
	wire								avs2vsi_ready		;

	wire								to32_valid			;
	wire								to32_frame_start	;
	wire								to32_frame_end		;
	wire								to32_field			;
	wire	[       DATA_BIT-1 : 0 ]	to32_pixel			;
	wire	[       SIZE_BIT-1 : 0 ]	to32_width			;
	wire	[       SIZE_BIT-1 : 0 ]	to32_height			;
	wire								to32_ready			;

	wire								div_valid			;
	wire								div_frame_start		;
	wire								div_frame_end		;
	wire								div_field			;
	wire	[       DATA_BIT-1 : 0 ]	div_data			;
	wire	[       SIZE_BIT-1 : 0 ]	div_width			;
	wire	[       SIZE_BIT-1 : 0 ]	div_height			;
	wire	[    FRM_NUM_BIT-1 : 0 ]	div_frm_number		;
	wire	[ PACKET_NUM_BIT-1 : 0 ]	div_pkt_number		;
	wire	[ PACKET_NUM_BIT-1 : 0 ]	div_ttl_pkt_count	;
	wire								div_ready			;

	wire								pkt_generate		;

	//	test signal
	reg		[               24 : 0 ]	input_cunter_ff		;
	reg		[               24 : 0 ]	internal_cunter_ff	;

// =============================================================================
// FUNCTION DESCRIPTION
// =============================================================================

	avb_pkt_asm_csr
		u_avb_pkt_asm_csr (
			.clk					( clk					) ,
			.rst_n					( rst_n					) ,

			.address_i				( avm_address_i			) ,
			.byteenable_i			( avm_byteenable_i		) ,
			.read_i					( avm_read_i			) ,
			.write_i				( avm_write_i			) ,
			.readdatavalid_o		( avm_readdatavalid_o	) ,
			.readdata_o				( avm_readdata_o		) ,
			.writedata_i			( avm_writedata_i		) ,
			.waitrequest_o			( avm_waitrequest_o		) ,

			.packet_generate_i		( pkt_generate			) ,

			.srst_o					( csr_srst				) ,
			.packet_size_o			( csr_packet_size		) ,
			.des_mac_adr_o			( csr_des_mac_adr		) ,
			.src_mac_adr_o			( csr_src_mac_adr		) ,
			.tag_o					( csr_tag				) ,
			.ether_type_o			( csr_ether_type		) ,
			.packet_transmit_o		( csr_packet_transmit	) 
		) ;


	avb_pkt_asm_avs2vsi
		u_avb_pkt_asm_avs2vsi (
			.clk				( clk					) ,
			.rst_n				( rst_n					) ,
			.srst				( csr_srst				) ,

			.valid_i			( valid_i				) ,
			.startofpacket_i	( startofpacket_i		) ,
			.endofpacket_i		( endofpacket_i			) ,
			.data_i				( data_i				) ,
			.ready_o			( ready_o				) ,

			.valid_o			( avs2vsi_valid			) ,
			.frame_start_o		( avs2vsi_frame_start	) ,
			.frame_end_o		( avs2vsi_frame_end		) ,
			.field_o			( avs2vsi_field			) ,
			.pixel_o			( avs2vsi_data			) ,
			.width_o			( avs2vsi_width			) ,
			.height_o			( avs2vsi_height		) ,
			.ready_i			( avs2vsi_ready			) ,
                                  
			.av_interlacing_o	( 						) 
		) ;

	avb_pkt_asm_to32
		u_avb_pkt_asm_to32 (
			.clk			( clk					) ,
			.rst_n			( rst_n					) ,
			.srst			( csr_srst				) ,

			.valid_i		( avs2vsi_valid			) ,
			.frame_start_i	( avs2vsi_frame_start	) ,
			.frame_end_i	( avs2vsi_frame_end		) ,
			.field_i		( avs2vsi_field			) ,
			.data_i			( avs2vsi_data			) ,
			.width_i		( avs2vsi_width			) ,
			.height_i		( avs2vsi_height		) ,
			.ready_o		( avs2vsi_ready			) ,
                              
			.valid_o		( to32_valid			) ,
			.frame_start_o	( to32_frame_start		) ,
			.frame_end_o	( to32_frame_end		) ,
			.field_o		( to32_field			) ,
			.data_o			( to32_pixel			) ,
			.width_o		( to32_width			) ,
			.height_o		( to32_height			) ,
			.ready_i		( to32_ready			) 
		) ;                   

	avb_pkt_asm_div
		u_avb_pkt_asm_div (
			.clk					( clk				) ,
			.rst_n					( rst_n				) ,
			.srst					( csr_srst			) ,

			.packet_size_i			( csr_packet_size	) ,

			.valid_i				( to32_valid		) ,
			.frame_start_i			( to32_frame_start	) ,
			.frame_end_i			( to32_frame_end	) ,
			.field_i				( to32_field		) ,
			.data_i					( to32_pixel		) ,
			.width_i				( to32_width		) ,
			.height_i				( to32_height		) ,
			.ready_o				( to32_ready		) ,

			.valid_o				( div_valid			) ,
			.frame_start_o			( div_frame_start	) ,
			.frame_end_o			( div_frame_end		) ,
			.field_o				( div_field			) ,
			.data_o					( div_data			) ,
			.width_o				( div_width			) ,
			.height_o				( div_height		) ,
			.frame_num_o			( div_frm_number	) ,
			.packet_number_o		( div_pkt_number	) ,
			.total_packet_count_o	( div_ttl_pkt_count	) ,
			.ready_i				( div_ready			) 
		) ;

	avb_pkt_asm_packet
		u_avb_pkt_asm_packet (
			.clk						( clk						) ,
			.rst_n						( rst_n						) ,
			.srst						( csr_srst					) ,

			.des_mac_adr_i				( csr_des_mac_adr			) ,
			.src_mac_adr_i				( csr_src_mac_adr			) ,
			.tag_i						( csr_tag					) ,
			.ether_type_i				( csr_ether_type			) ,

			.local_sync_256ns_pulse_i	( local_sync_256ns_pulse_i	) ,
			.master_clock_sec_i			( master_clock_sec_i		) ,
			.master_clock_ns_i			( master_clock_ns_i			) ,

			.packet_transmit_i			( csr_packet_transmit		) ,
			.packet_generate_o			( pkt_generate				) ,

			.valid_i					( div_valid					) ,
			.frame_start_i				( div_frame_start			) ,
			.frame_end_i				( div_frame_end				) ,
			.field_i					( div_field					) ,
			.data_i						( div_data					) ,
			.width_i					( div_width					) ,
			.height_i					( div_height				) ,
			.frame_num_i				( div_frm_number			) ,
			.packet_number_i			( div_pkt_number			) ,
			.total_packet_count_i		( div_ttl_pkt_count			) ,
			.ready_o					( div_ready					) ,

			.tx_valid_o					( tx_valid_o				) ,
			.tx_startofpacket_o			( tx_startofpacket_o		) ,
			.tx_endofpacket_o			( tx_endofpacket_o			) ,
			.tx_data_o					( tx_data_o					) ,
			.tx_mod_o					( tx_mod_o					) ,
			.tx_err_o					( tx_err_o					) ,
			.tx_ready_i					( tx_ready_i				) 
		) ;

	always @ ( posedge clk or negedge rst_n ) begin
		if ( !rst_n ) begin
			input_cunter_ff	<= 25'd1 ;
		end
		else if ( csr_srst ) begin
			input_cunter_ff	<= 25'd1 ;
		end
		else begin
			if ( valid_i && startofpacket_i && ready_o ) begin
				input_cunter_ff	<= 25'd1 ;
			end
			else if ( valid_i && ready_o ) begin
				input_cunter_ff	<= input_cunter_ff + 1'b1 ;
			end
		end
	end

	always @ ( posedge clk or negedge rst_n ) begin
		if ( !rst_n ) begin
			internal_cunter_ff	<= 25'd1 ;
		end
		else if ( csr_srst ) begin
			internal_cunter_ff	<= 25'd1 ;
		end
		else begin
			if ( avs2vsi_valid && avs2vsi_frame_start && avs2vsi_ready ) begin
				internal_cunter_ff	<= 25'd1 ;
			end
			else if ( avs2vsi_valid && avs2vsi_ready ) begin
				internal_cunter_ff	<= internal_cunter_ff + 1'b1 ;
			end
		end
	end



endmodule
`default_nettype wire